# -*- coding: iso-8859-1 -*-
"""
    MoinMoin - TokenSupport library

    @copyright: 2013 by Paul Boddie <paul@boddie.org.uk>
    @license: GNU GPL (v2 or later), see COPYING.txt for details.
"""

import re

identifier_expr = re.compile(
    """(?P<non_literal>[^'" ]+)"""
    "|"
    "(?P<spaces> +)"
    "|"
    "(?P<literal1>'[^']*')"
    "|"
    '(?P<literal2>"[^"]*")'
    )

def getIdentifiers(s, doubling=False):

    """
    Return 's' containing space-separated quoted identifiers, parsed into
    regions that hold the individual identifiers. The optional 'doubling'
    argument can be used to support convenient quote doubling to reproduce
    single quote characters.

    Quoting of identifiers can be done using the single-quote and double-quote
    characters in order to include spaces within identifiers. For example:

       'contains space'
    -> contains space                   (a single identifier)

    Where one kind of quote (or apostrophe) is to be included in an identifier,
    the other quoting character can be used to delimit the identifier. For
    example:

       "Python's syntax"
    -> Python's syntax                  (a single identifier)

    Where the 'doubling' argument is set to a true value, a quote character can
    be doubled to include it in an identifier. For example:

       Python''s syntax
    -> Python's syntax                  (a single identifier)

    Where a mixture of quotes is required in a single identifier, adjacent
    quoted regions can be used. For example:

       "Python's "'"intuitive" syntax'
    -> "Python's "                      (region #1)
     + '"intuitive" syntax'             (region #2)
    -> Python's "intuitive" syntax      (a single identifier)

    Where unquoted regions are adjacent to quoted regions, the regions are
    combined. For example:

       "Python's "intuitive" syntax"
    -> "Python's "                      (region #1)
     + intuitive                        (region #2)
     + " syntax"                        (region #3)
    -> Python's intuitive syntax        (a single identifier)
    """

    regions = []
    in_literal = False

    for match in identifier_expr.finditer(s):
        non_literal, spaces, literal1, literal2 = match.groups()

        identifier = None

        # Spaces prevent continuation of identifier regions.

        if spaces:
            in_literal = False

        # Unquoted regions contribute to the current identifier.

        if non_literal and non_literal.strip():
            identifier = non_literal.strip()

        # Quoted regions also contribute to the current identifier.

        for s in (literal1, literal2):
            if s is not None:

                # Either strip the quoting or for empty regions, adopt the
                # quote character.

                if not doubling or len(s) > 2:
                    identifier = s[1:-1]
                elif doubling:
                    identifier = s[0]

        # Either continue or add an identifier, and indicate possible
        # continuation.

        if identifier:
            if in_literal:
                regions[-1] += identifier
            else:
                regions.append(identifier)
            in_literal = True

    return regions

# vim: tabstop=4 expandtab shiftwidth=4
